/**	code: 1-Nov-2000
***     Indirect estimates of the conditional Factor model with garch (world and two market factors)
***     of exchange rates in the East Asian Crisis
***
***     Data are daily from 2/1/95 to 31/8/98
***
***     Order of variables is:
***
***       
***			KOREA
***			INDONESIA
***			THAILAND
***			MALAYSIA
***			US
***			AUSTRALIA
***			JAPAN
***			SINGAPORE
***
***     Complete model
***
***
***		markets are ordered: Interest rates, Currency then Equity
***
***		THIS ONE HAS 2 MARKETS EQUITY AND CURRENCY MARKETS
		SPILLOVERS BETWEEN MARKETS ONLY 


		THESE RESULTS FROM GAUSS 5.0

		new auxiliary model developed here.

***
***			b denotes bonds, c denotes currencies, e denotes equities
***
**/

library optmum, pgraph,maxlik;
#include optmum.ext;
#include maxlik.ext;
optset;
gausset;
graphset;
maxset;


format 12,5;
/*output file = c:\gmm.out reset;*/

rndseed 123457;

@********************************DATA *****************************************@



load rt[956,5]=c:\diskette\mardi\multi\fxdata2.txt;	@exchange rate data - Korea, Indonesia, Thailand, Malaysia, Australia@

erate=ln(rt);
erate=100*(trimr(erate,1,0)-trimr(erate,0,1));

erate=erate[652:955,.];


y1t = erate - meanc(erate)';
/*y1t=erate;*/


/**		Filter ar(1) from Korea		**/

y = trimr(y1t[.,1],1,0);
x = trimr(y1t[.,1],0,1);

y = y - (y/x)*x;

y1t = y ~ trimr(y1t[.,2:cols(y1t)],1,0);



load rt1[956,8]=c:\diskette\mardi\multi\irateasia.txt;	@interest rate data@
rt1=100*rt1/365;
rt1=rt1[.,5 1 2 3 4 6];
rt1=rt1[652:955,.];
y2t = rt1 - meanc(rt1)';
/*y2t=rt1;*/

load rt1[956,7]=c:\diskette\mardi\multi\sdata.txt;		@equity data@
stock=ln(rt1);
stock=100*(trimr(stock,1,0)-trimr(stock,0,1));
stock=stock[.,6 2 1 4 3 7];   @US, Korea, Indonesia, Thailand, Malaysia, Australia@
stock=stock[652:955,.];
y3t = stock - meanc(stock)';
/*y3t=stock;*/
y3t=trimr(y3t,1,0);


tt = seqa(1997,1/260,303);


yt = y1t~y3t;   /**		currency and equities    **/



@------------------------------------------------------------------------------------------------------------------------------------------
				Calculate the auxiliary model of the data
--------------------------------------------------------------------------------------------------------------------------------------------@


itype = 0.0;


	{va,ve} = eigrs2(corrx(yt));

	loadings = ve[.,(cols(ve)-2):cols(ve)] ; @---  choose best three principal components   ---@


	ytfact = yt*loadings ;  

	y = trimr(yt,1,0);
	x = trimr(ytfact,0,1);
	bhat1 = y/x;
	u1 = y - x*bhat1;
	g1 = u1 *~ x;


y0 = (u1-meanc(u1)') *~ (u1-meanc(u1)');		@cov matrix calc as 1^2,1*2,1*3,1*4,2*1,2^2,2*3,2*4.....4^4@
y0   = y0[.,vech( reshape(seqa(1,1,cols(u1)^2),cols(u1),cols(u1)) )] ;		@take the lower diagonal of y2@
bhat0 = meanc(y0);
g0 = y0 - bhat0';	


	
	pc1 = ytfact[.,3];
	dpc1 = trimr(pc1,1,0) - trimr(pc1,0,1);

	
	pc2 = ytfact[.,2];
	dpc2 = trimr(pc2,1,0) - trimr(pc2,0,1);

	
	pc3 = ytfact[.,1];
	dpc3 = trimr(pc3,1,0) - trimr(pc3,0,1);


	zt = (dpc1.^2 - meanc(dpc1.^2))/stdc(dpc1.^2);
	z2 = trimr(zt,2,0).*trimr(zt,1,1) ~ trimr(zt,2,0).*trimr(zt,0,2);
	bhat2 = meanc(z2);
	g2 = z2 - bhat2';



	zt = (dpc2.^2 - meanc(dpc2.^2))/stdc(dpc2.^2);
	z2 = trimr(zt,2,0).*trimr(zt,1,1) ~ trimr(zt,2,0).*trimr(zt,0,2);
	bhat3 = meanc(z2);
	g3 = z2 - bhat3';


	zt = (dpc3.^2 - meanc(dpc3.^2))/stdc(dpc3.^2);
	z2 = trimr(zt,2,0).*trimr(zt,1,1) ~ trimr(zt,2,0).*trimr(zt,0,2);
	bhat4 = meanc(z2);
	g4 = z2 - bhat4';

    

bhat =  vec(bhat1) | vec(bhat0)  |  bhat2 | bhat3 | bhat4;

g = trimr( g1 ~ g0 , 2, 0)   ~ g2 ~ g3 ~ g4;






m = cols(g);
i0  = g'g;

    

lmax = 5.0;
l=1;
    do until l>lmax;
       wtau = g[(l+1):rows(g),.]'g[1:(rows(g)-l),.];			@the squares and cross products@
       i0   = i0 + (1.0-l/(lmax+1))*(wtau + wtau');			@weighted sum of the squares and cross products@	
       l    = l + 1;		
    endo;

i0 = i0/rows(g);									@take the average@

sigma = inv( i0 );								@take the inverse@
nobs=rows(yt);




btilda0=0.0; btilda1=0.0;btilda2=0.0;btilda3=0.0;btilda4=0.0;btilda=0.0;

/*      Parameters for indirect estimation                  */
    t   = nobs;            /*  Sample size                  */
    h   = 100;               /*  Number of simulation paths   */
    n   = 100+h*t;               /*  Length of simulated series   */


	
@exchange market effects@

@country and market specific effect@
	ac0=1;
	ac1=1;
	ac2=1;
	ac3=1;
	ac4=1;
	ac5=1;

@exchange market effect@
	bc1=1;
	bc2=1;
	bc3=1;
	bc4=1;
	bc5=1;

@country effect@
	cc0=1;
	cc1=1;
	cc2=1;
	cc3=1;
	cc4=1;
	cc5=1;

@world effect@
	dc1=1;
	dc2=1;
	dc3=1;
	dc4=1;
	dc5=1;

@ parameters on equity markets@
@country and market specific effect@
	ae0=1;
	ae1=1;
	ae2=1;
	ae3=1;
	ae4=1;
	ae5=1;

@equity market effect@
	be0=1;
	be1=1;
	be2=1;
	be3=1;
	be4=1;
	be5=1;

@country effect@
	ce0=1;
	ce1=1;
	ce2=1;
	ce3=1;
	ce4=1;
	ce5=1;

@world effect@
	de0=1;
	de1=1;
	de2=1;
	de3=1;
	de4=1;
	de5=1;

@market spillovers - from each market to the other@
@currency market@
	@spillover to currency from equity@
	fce1=1;
	fce2=1;
	fce3=1;
	fce4=1;
	fce5=1;
@equity market@
	@spillover to equity from currency@ 
	fec0=1;
	fec1=1;
	fec2=1;
	fec3=1;
	fec4=1;
	fec5=1;

@time zone spillovers adjustments for equity markets@
	dm1=0.01;
	dm2=0.01;
	dm3=0.01;
	dm4=0.01;
	dm5=0.01;

@contagion@
	@contagion to equity from currency@ 
	gec0=1;
	gec1=1;
	gec2=1;
	gec3=1;
	gec4=1;
	gec5=1;
	@contagion to currency from equity@
	gce1=1;
	gce2=1;
	gce3=1;
	gce4=1;
	gce5=1;

@AR structure@
	@AR coefficient on the country factors@	
	pc0=0;
	pc1=0;
	pc2=0;
	pc3=0;
	pc4=0;
	pc5=0;



	@AR coefficients on the two market factors and the world factor@
	pmc=0;
	pme=0;
	pmw=0;	


alphamc = 0.0;
betamc   = 0.0;

alphame = 0.0;
betame   = 0.0;

alphaw = 0.0;
betaw   = 0.0;


m=zeros(66,1);
mt=zeros(t,66);

            ewt = zeros(n,1);
            e0t = zeros(n,1);
            e1t = zeros(n,1);
            e2t = zeros(n,1);
            e3t = zeros(n,1);
            e4t = zeros(n,1);

	h0 = 0.1; 

            hwt = h0*ones(n,1);
            h0t = h0*ones(n,1);
            h1t = h0*ones(n,1);
            h2t = h0*ones(n,1);
            h3t = h0*ones(n,1);
            h4t = h0*ones(n,1);
  

            fwt = zeros(n,1);
            f0t = zeros(n,1);
            f1t = zeros(n,1);
            f2t = zeros(n,1);
            f3t = zeros(n,1);
            f4t = zeros(n,1);


	hmct = zeros(n,1);
	hmet = zeros(n,1);
	hwt = zeros(n,1);

	emct = zeros(n,1);	
	emet = zeros(n,1);
	ewt   = zeros(n,1);

	emetl =  zeros(n,1);
	emctl =  zeros(n,1);

	mct=zeros(n+1,1);
	met=zeros(n+1,1);
	wt=zeros(n+1,1);

	mctl=zeros(n,1);
	metl=zeros(n,1);

/*      Generate errors to simulate the true model      */
    uwt = rndn(n,1);      	/** world factor  **/

    umct = rndn(n+1,1);		/** currency market factor  **/
/*    umctl = trimr(umct,0,1);
    umct = trimr(umct,1,0);    */

    umet = rndn(n+1,1);		/** equity market factor  **/
/*    umetl = trimr(umet,0,1);
    umet = trimr(umet,1,0);    */
    


    uz0t = rndn(n,1);		/** country numeraire factor  **/
    uz1t = rndn(n,1);		/** country 1 factor   **/
    uz2t = rndn(n,1);
    uz3t = rndn(n,1);
    uz4t = rndn(n,1);
    uz5t = rndn(n,1);		/** country 5 factor   **/


    ucc0t = rndn(n,1);		/** idiosyncratic for country 1 and currency market   **/
    ucc1t = rndn(n,1);		/** idiosyncratic for country 2 and currency market    **/
    ucc2t = rndn(n,1);
    ucc3t = rndn(n,1);
    ucc4t = rndn(n,1);
    ucc5t = rndn(n,1);		/** idiosyncratic for country 5 and currency market   **/

    uce0t = rndn(n+1,1);		/** idiosyncratic for country numeraire and equity market   **/
    uce0tlg=trimr(uce0t,0,1);
    uce0t=trimr(uce0t,1,0);
    uce1t = rndn(n,1);		/** idiosyncratic for country 1 and equity market                   **/
    uce2t = rndn(n,1);
    uce3t = rndn(n,1);
    uce4t = rndn(n,1);
    uce5t = rndn(n,1);		/** idiosyncratic for country 5 and equity market                      **/




   uT=rndn(n,1);		@errors for Thailand@

    ytsim = 0.0;




/*      The objective function to be minimized          */
    proc mbar(b);
        local i,k,dq,ftsim,htsim,etsim,a0,b0,r0,ytsimt;

        dq = zeros(1,rows(vec(bhat)));			@h is the number of simulations @

        /*  Simulate the true model     */

  	ac0=b[1];
	ac1=b[2];
	ac2=b[3];
	ac3=b[4];
	ac4=b[5];
	ac5=b[6];

	bc1=b[7];
	bc2=b[8];
	bc3=b[9];
	bc4=b[10];
	bc5=b[11];

	cc0=b[12];
	cc1=b[13];
	cc2=b[14];
	cc3=b[15];
	cc4=b[16];
	cc5=b[17];

	dc1=b[18];
	dc2=b[19];
	dc3=b[20];
	dc4=b[21];
	dc5=b[22];

  	ae0=b[23];
	ae1=b[24];
	ae2=b[25];
	ae3=b[26];
	ae4=b[27];
	ae5=b[28];

	be0=b[29];
	be1=b[30];
	be2=b[31];
	be3=b[32];
	be4=b[33];
	be5=b[34];

	ce0=b[35];
	ce1=b[36];
	ce2=b[37];
	ce3=b[38];
	ce4=b[39];
	ce5=b[40];

	de0=b[41];
	de1=b[42];
	de2=b[43];
	de3=b[44];
	de4=b[45];
	de5=b[46];


	fce1=b[47];
	fce2=b[48];
	fce3=b[49];
	fce4=b[50];
	fce5=b[51];

	fec0=b[52];
	fec1=b[53];
	fec2=b[54];
	fec3=b[55];
	fec4=b[56];
	fec5=b[57];


if itype == 0;

	betamc = cdfn(b[58]);
	betame = cdfn(b[59]);
	betaw   = cdfn(b[60]);


	alphamc = cdfn(b[58] + abs(b[61]) ) - betamc;
	alphame = cdfn(b[59] + abs(b[62]) ) - betame; 
	alphaw = cdfn(b[60] + abs(b[63]) ) - betaw; 

	pmc=tanh(b[75]);
	pme=tanh(b[76]);
	pmw=tanh(b[77]);

    else;       /**     for computing standard errors   **/

	betamc = (b[58]);
	betame = (b[59]);
	betaw   = (b[60]);


	alphamc = b[61];
	alphame = b[62]; 
	alphaw  = b[63]; 

	pmc=(b[75]);
	pme=(b[76]);
	pmw=(b[77]);

endif;


	gce1=b[64];
	gce2=b[65];
	gce3=b[66];
	gce4=b[67];
	gce5=b[68];

	gec0=b[69];
	gec1=b[70];
	gec2=b[71];
	gec3=b[72];
	gec4=b[73];
	gec5=b[74];


	dm1=b[78];
	dm2=b[79];
	dm3=b[80];
	dm4=b[81];
	dm5=b[82];

	hmct = recsercp( alphamc*(0.0 | trimr(umct.^2,0,1) ) + betamc , 1-alphamc-betamc);
	hmet = recsercp( alphame*(0.0 | trimr(umet.^2,0,1) ) + betame , 1-alphame-betame);
	hwt = recsercp( alphaw*(0.0 | trimr(uwt.^2,0,1) ) + betaw , 1-alphaw-betaw);

	emct = umct.*sqrt(hmct);	
	emet = umet.*sqrt(hmet);
	ewt   = uwt.*sqrt(hwt);

	mct=recserar(emct,0.0,pmc);
	met=recserar(emet,0.0,pme);
	wt=recserar(ewt,0.0,pmw);

	emct=trimr(emct,1,0);
	emet=trimr(emet,1,0);	

	mctl=trimr(mct,0,1);
	mct=trimr(mct,1,0);
	metl=trimr(met,0,1);
	met=trimr(met,1,0);



/*
	emetl = trimr(emet,0,1);
	emet  = trimr(emet,1,0);

	emctl = trimr(emct,0,1);
	emct  = trimr(emct,1,0);
*/





	ytsim =         ac1*ucc1t + ac0*ucc0t + bc1*mct + cc0*uz0t + cc1*uz1t + dc1*wt  +  fce1*metl + gce1*emet;
	ytsim = ytsim ~ ac2*ucc2t + ac0*ucc0t + bc2*mct + cc0*uz0t + cc2*uz2t + dc2*wt  +  fce2*metl + gce2*emet;
	ytsim = ytsim ~ ac3*ucc3t + ac0*ucc0t + bc3*mct + cc0*uz0t + cc3*uz3t + dc3*wt  +  fce3*metl + gce3*emet ; 
	ytsim = ytsim ~ ac4*ucc4t + ac0*ucc0t + bc4*mct + cc0*uz0t + cc4*uz4t + dc4*wt  +  fce4*metl + gce4*emet; 
	ytsim = ytsim ~ ac5*ucc5t + ac0*ucc0t + bc5*mct + cc0*uz0t + cc5*uz5t + dc5*wt  +  fce5*metl + gce5*emet;

	ytsim = ytsim ~ ae0*uce0t + be0*met + ce0*uz0t + de0*wt +  fec0*mctl + gec0*emct;
	ytsim = ytsim ~ ae1*uce1t + be1*met + ce1*uz1t + de1*wt +  fec1*mctl + gec1*emct + dm1*uce0tlg;
	ytsim = ytsim ~ ae2*uce2t + be2*met + ce2*uz2t + de2*wt +  fec2*mctl + gec2*emct + dm2*uce0tlg;
	ytsim = ytsim ~ ae3*uce3t + be3*met + ce3*uz3t + de3*wt +  fec3*mctl + gec3*emct + dm3*uce0tlg;
	ytsim = ytsim ~ ae4*uce4t + be4*met + ce4*uz4t + de4*wt +  fec4*mctl + gec4*emct + dm4*uce0tlg;
	ytsim = ytsim ~ ae5*uce5t + be5*met + ce5*uz5t + de5*wt +  fec5*mctl + gec5*emct + dm5*uce0tlg;



		@develop the criteria for comparison with the true data@


    yt = ytsim;


	ytfact = yt*loadings ;  

	y = trimr(yt,1,0);
	x = trimr(ytfact,0,1);
	btilda1 = y/x;
	u1 = y - x*bhat1;
	g1 = u1 *~ x;


y0 = (u1-meanc(u1)') *~ (u1-meanc(u1)');		@cov matrix calc as 1^2,1*2,1*3,1*4,2*1,2^2,2*3,2*4.....4^4@
y0   = y0[.,vech( reshape(seqa(1,1,cols(u1)^2),cols(u1),cols(u1)) )] ;		@take the lower diagonal of y2@
btilda0 = meanc(y0);
g0 = y0 - bhat0';	


	pc1 = ytfact[.,3];
	dpc1 = trimr(pc1,1,0) - trimr(pc1,0,1);

	
	pc2 = ytfact[.,2];
	dpc2 = trimr(pc2,1,0) - trimr(pc2,0,1);

	
	pc3 = ytfact[.,1];
	dpc3 = trimr(pc3,1,0) - trimr(pc3,0,1);


	zt = (dpc1.^2 - meanc(dpc1.^2))/stdc(dpc1.^2);
	z2 = trimr(zt,2,0).*trimr(zt,1,1) ~ trimr(zt,2,0).*trimr(zt,0,2);
	btilda2 = meanc(z2);
	g2 = z2 - bhat2';



	zt = (dpc2.^2 - meanc(dpc2.^2))/stdc(dpc2.^2);
	z2 = trimr(zt,2,0).*trimr(zt,1,1) ~ trimr(zt,2,0).*trimr(zt,0,2);
	btilda3 = meanc(z2);
	g3 = z2 - bhat3';


	zt = (dpc3.^2 - meanc(dpc3.^2))/stdc(dpc3.^2);
	z2 = trimr(zt,2,0).*trimr(zt,1,1) ~ trimr(zt,2,0).*trimr(zt,0,2);
	btilda4 = meanc(z2);
	g4 = z2 - bhat4';

    btilda =  vec(btilda1) | vec(btilda0)  |  btilda2 | btilda3 | btilda4;

  
    g = trimr( g1 ~ g0 , 2, 0)   ~ g2 ~ g3 ~ g4;




		@make the comparison@

            	dq[1,.] =  meanc( g )';


        retp( meanc(dq) );				@ how do the simulations compare with the actual data - minimise @

    endp;



    proc fobj(b);						@this is for the standard errors@
	local mb;
	mb = mbar(b);
        retp( mb'sigma*mb );			@ weighted sum of the simulations compared with actual data - minimise@
    endp;




/*      Estimate the indirect model using simulated data   */


_opalgr  = 2;
__output = 2;
_opgtol  = 0.001;



bstart=0.01*ones(82,1);



/**  h=100  from Vance**/

bstart = {
0.263714388160 
1.58399402039 
3.14764373553 
-0.182385954785 
-0.0962960010346 
0.0752074515224 
-0.597803765015 
-0.866123529694 
-0.234115782132 
0.0354048681037 
-0.264538251592 
-0.0590502142719 
-0.437786255963 
0.212698046621 
-1.11247687074 
0.721707540352 
0.547493286173 
-0.249765863594 
-1.44530575516 
-0.840463749035 
-0.918929675862 
-0.280500776560 
-0.654719505109 
-1.66466871558 
-0.353476161040 
1.57126449779 
-1.75128201557 
0.177295007545 
-0.304358112140 
-1.59331401276 
0.328591467758 
-0.443657500015 
-0.604630321322 
-0.311278017732 
0.773722556199 
1.77720342985 
1.53753814354 
0.326626792452 
-0.301814218455 
-0.0302955710923 
-0.0969842177546 
0.112604742825 
0.837868731091 
0.308145921363 
0.994685729284 
-0.0310311694006 
0.842937589666 
1.83733605739 
0.102205708842 
0.251409444394 
-0.121509785374 
0.0979684557858 
1.39873760402 
1.43620599346 
1.63100610622 
1.50826010532 
0.228066494767 
0.922628510556 
-0.0131236054775 
1.04637696436 
1.53264484935 
1.61593382283 
0.297157609920 
-0.562856214810 
1.40936889244 
0.217472815383 
0.930896970305 
0.256494545154 
0.380568350515 
0.241810289404 
0.386440262963 
-0.0420192357090 
0.539741424779 
0.338515080716 
0.445258479777 
0.262636551089 
0.0605630869336 
-0.0642317989169 
-0.542711444830 
-0.465081582197 
-0.595109875932 
-0.880136602002 
};


/** Compare correlations of actual and simulated data    **/


tmp1=corrx(yt[.,7 8 9 10 11 6 1 2 3 4 5]);  /**     Order of countries consistent with tables in paper  **/
call mbar(b);tmp2=corrx(yt[.,7 8 9 10 11 6 1 2 3 4 5]);
vech(tmp1)~vech(tmp2); 
print "";


itype = 0.0;

/*
{b,f,g_deriv,retcode} = optprt(optmum(&fobj,bstart));		@do the indirect estimation@
*/

b=bstart';


qmin = t*fobj(bstart);
nmom  = rows(sigma);
npara = rows(b);
dof = nmom - npara;

print "";
print "";
print "GMM Results";
print "***********";
print "The value of the objective function is " qmin;
if dof > 0.0; print "Overidentifying restriction test p-value is " cdfchic(qmin,dof); 
print "     With degrees of freedom =" dof; endif;




/**     Compute standard errors       **/

    itype = 1.0;

	b[58] = betamc;
	b[59] = betame;
	b[60] = betaw;


	b[61] = alphamc;
	b[62] = alphame; 
	b[63] = alphaw; 

	b[75] = pmc;
	b[76] = pme;
	b[77] = pmw;



    g0   = gradp(&mbar,b);
    vcov = inv(g0'inv(i0)*g0)/t;
    se   = sqrt(diag(vcov));



format /rd 12,3;
print "";
print "   Estimates    std errors       t-stats   " 
b~se~(b./se);


print "";
print "";


/**     Compute diagnostics     **/

call mbar(b);           /**     Generate the auxiliary scores evaluated using simulated data ie g       **/

d = bhat - btilda;      /**     Difference in auxiliary estimates using actual and simulated data resp. **/

id = g'g/t;             /**     Information matrix of differences in auxiliary parameter estimates      **/

id = i0;

/**     Test that the differences are all zero  **/

r = eye(105);

q = zeros(105,1);

teststat = t*(r*d - q)'inv(r*id*r')*(r*d - q);

print "Consistency test that the differences between bhat and btilda are zero for all parameters";
print "     Statistic       DOF         p-value";
teststat~rows(r)~cdfchic(teststat,rows(r));
print "";


/**     Test that the differences are zero for contemporaneous estimates  **/

r = zeros(66,33) ~ eye(66) ~ zeros(66,6);

q = zeros(66,1);

teststat = t*(r*d - q)'inv(r*id*r')*(r*d - q);

print "Consistency test that the differences between bhat and btilda are zero for contemporaneous auxiliary parameters";
print "     Statistic       DOF         p-value";
teststat~rows(r)~cdfchic(teststat,rows(r));
print "";



/**     Test that the differences are zero for ar estimates  **/

r = eye(33) ~ zeros(33,72);

q = zeros(33,1);

teststat = t*(r*d - q)'inv(r*id*r')*(r*d - q);

print "Consistency test that the differences between bhat and btilda are zero for ar auxiliary parameters";
print "     Statistic       DOF         p-value";
teststat~rows(r)~cdfchic(teststat,rows(r));
print "";




/**     Test that the differences are zero for garch estimates  **/

r = zeros(6,99) ~ eye(6);

q = zeros(6,1);

teststat = t*(r*d - q)'inv(r*id*r')*(r*d - q);

print "Consistency test that the differences between bhat and btilda are zero for garch auxiliary parameters";
print "     Statistic       DOF         p-value";
teststat~rows(r)~cdfchic(teststat,rows(r));
print "";



stop;



/**     Construct parameters     

    delta         World parameters               
    lam           Market parameters              
    omega         Country parameters             
    d             Spillover parameters           
    gam           Contagion parameters 
    phi           Unique in country market parameters 
    kappa         Time zone effect parameters     
**/


@its world, market, country0, country,spillover, contagion, unique-0, unique, timezone@
    delta = b[18:22] |  b[41:46];
    lam   = ( b[7:11] | zeros(6,1) ) ~ ( zeros(5,1) | b[29:34] );
    omega = ( b[12]*ones(5,1) ~ diagrv(eye(5),b[13:17]) ) | diagrv(eye(6),b[35:40]);  
    d     = ( b[47:51] | zeros(6,1) ) ~ ( zeros(5,1) | b[52:57] );
    gam   = ( b[64:68] | zeros(6,1) ) ~ (zeros(5,1) | b[69:74]);
    phi   = ( b[1]*ones(5,1) | zeros(6,1) ) ~ diagrv(eye(11),b[2:6]|b[23:28]);
    kappa = zeros(6,1) | b[78:82];


/**     Adjust parameters for long-run calculations     **/

@check Vance's old stuff@
    olddelta=delta;
    oldd=d;

    delta = delta./sqrt(1-pmw^2);       /** This new  **/
    lam   = lam./sqrt(1 - (pmc~pme).^2);
    d     = d./sqrt(1 - (pmc~pme).^2);  /** This is new  **/

    /**     Combine matrices and compute long-run decomposition       **/

    total = delta ~ lam ~ omega ~ d ~ gam ~ phi ~ kappa;  

    decomp = 100*total.^2 ./ sumc( (total.^2)');

rc=zeros(8,1);
for k(1,5,1);
    r1=decomp[k,.]';
    r2=selif(r1,r1.gt 0);
    rc=rc~r2;
endfor;
rc=rc[.,2:cols(rc)];

re=decomp[6,.]';
re=selif(re,re.gt 0);
re=re|0;
for k(7,11,1);
    r1=decomp[k,.]';
    r2=selif(r1,r1.gt 0);
    re=re~r2;
endfor;


/**         Compute standard errors of variance decompositions      **/

proc dc(b);

    local pmc,pme,pmw,delta,lam,omega,d,gam,phi,kappa,total,decomp;

	pmc = (b[75]);
	pme = (b[76]);
	pmw = (b[77]);

    delta = b[18:22] |  b[41:46];
    lam   = ( b[7:11] | zeros(6,1) ) ~ ( zeros(5,1) | b[29:34] );
    omega = ( b[12]*ones(5,1) ~ diagrv(eye(5),b[13:17]) ) | diagrv(eye(6),b[35:40]);  
    d     = ( b[47:51] | zeros(6,1) ) ~ ( zeros(5,1) | b[52:57] );
    gam   = ( b[64:68] | zeros(6,1) ) ~ (zeros(5,1) | b[69:74]);
    phi   = ( b[1]*ones(5,1) | zeros(6,1) ) ~ diagrv(eye(11),b[2:6]|b[23:28]);
    kappa = zeros(6,1) | b[78:82];

    delta = delta./sqrt(1-pmw^2);       /** This new  **/
    lam   = lam./sqrt(1 - (pmc~pme).^2);
    d     = d./sqrt(1 - (pmc~pme).^2);  /** This is new  **/

    total = delta ~ lam ~ omega ~ d ~ gam ~ phi ~ kappa;  

    decomp = 100*total.^2 ./ sumc( (total.^2)');

    retp( vec(decomp) );

endp;

v0 = dc(b);

dv0 = gradp(&dc,b);

sev  = sqrt( diag( dv0*vcov*dv0') ) ;   /**variance decomp standard errors  **/

print "Variance decomposition (%) and standard errors";
print "     Estimate  Standard error";
selif(v0~sev,v0 .gt 0.0);

print "";
print "";
stop;

format /rdt 2,3;

dlab="Korea "$|"Indon "$|"Thai "$|"Malay "$|"Aust ";
print "Currency markets"; 
dlab';
rc;
print "";

dlab="US"$|"Korea "$|"Indon "$|"Thai "$|"Malay "$|"Aust ";
print "Equity markets"; 
dlab';
re;






stop;



/********************

h=100


===============================================================================
 OPTMUM Version 3.1.10                                     1/17/2003   8:50 am
===============================================================================

return code =    0
normal convergence

Value of objective function        0.053189

Parameters    Estimates    Gradient
-----------------------------------------
P01              0.2637       -0.0003
P02              1.5840       -0.0002
P03              3.1476        0.0000
P04             -0.1824       -0.0001
P05             -0.0963        0.0002
P06              0.0752        0.0002
P07             -0.5978       -0.0003
P08             -0.8661        0.0000
P09             -0.2341        0.0003
P10              0.0354        0.0001
P11             -0.2645       -0.0003
P12             -0.0591        0.0009
P13             -0.4378        0.0002
P14              0.2127        0.0000
P15             -1.1125       -0.0001
P16              0.7217       -0.0005
P17              0.5475        0.0002
P18             -0.2498        0.0002
P19             -1.4453       -0.0001
P20             -0.8405        0.0000
P21             -0.9189        0.0003
P22             -0.2805        0.0004
P23             -0.6547        0.0003
P24             -1.6647       -0.0001
P25             -0.3535       -0.0001
P26              1.5713        0.0002
P27             -1.7513       -0.0001
P28              0.1773        0.0005
P29             -0.3044        0.0001
P30             -1.5933        0.0000
P31              0.3286        0.0000
P32             -0.4437        0.0000
P33             -0.6046        0.0000
P34             -0.3113        0.0000
P35              0.7737       -0.0001
P36              1.7772        0.0001
P37              1.5375        0.0000
P38              0.3266       -0.0002
P39             -0.3018       -0.0001
P40             -0.0303        0.0001
P41             -0.0970       -0.0004
P42              0.1126        0.0001
P43              0.8379        0.0001
P44              0.3081       -0.0002
P45              0.9947        0.0000
P46             -0.0310       -0.0004
P47              0.8429        0.0000
P48              1.8373        0.0000
P49              0.1022        0.0000
P50              0.2514       -0.0001
P51             -0.1215        0.0001
P52              0.0980       -0.0002
P53              1.3987        0.0000
P54              1.4362        0.0001
P55              1.6310        0.0002
P56              1.5083       -0.0001
P57              0.2281       -0.0002
P58              0.9226       -0.0001
P59             -0.0131       -0.0001
P60              1.0464        0.0000
P61              1.5326       -0.0004
P62              1.6159        0.0002
P63              0.2972        0.0000
P64             -0.5629        0.0000
P65              1.4094       -0.0001
P66              0.2175        0.0000
P67              0.9309        0.0000
P68              0.2565       -0.0005
P69              0.3806        0.0000
P70              0.2418        0.0001
P71              0.3864        0.0000
P72             -0.0420        0.0000
P73              0.5397        0.0000
P74              0.3385        0.0003
P75              0.4453        0.0000
P76              0.2626       -0.0001
P77              0.0606        0.0001
P78             -0.0642        0.0000
P79             -0.5427        0.0002
P80             -0.4651       -0.0001
P81             -0.5951        0.0001
P82             -0.8801       -0.0009

Number of iterations      401
Minutes to convergence  1032.00807
equity market
                 Own   Market     Country   World     Spillover currency   Contagion	Timezone
US                     33.169       7.6738       46.323      0.72782      0.89990       11.207 
Korea                24.980       24.500       28.472      0.11430       21.370      0.52709     0.037191 
Indonesia          1.9992       1.8495       37.825       11.233       39.991       2.3894       4.7127 
Thailand            39.048       3.3328       1.6873       1.5018       50.981     0.027925       3.4210 
Malaysia          38.623       4.9287       1.1471       12.460       34.713       3.6687       4.4600 
Australia          2.8857       9.5229     0.084258     0.088400       5.7860       10.520       71.113 
currency market
               Own   	Numeraireown    Market     Country  Numerairecountry  World  Spillover equity
Korea                57.069       1.5818       8.7022       4.3593     0.079312       1.4189       19.583       7.2059 
Indonesia          52.160      0.36613       4.2281      0.23817     0.018357       10.997       21.535       10.457 
Thailand             1.5337       3.2065       2.7054       57.061      0.16077       32.568      0.58359       2.1806 
Malaysia           0.38765       2.9073     0.056100       21.774      0.14577       35.301       3.2017       36.226 
Australia           0.91863       11.295       12.168       48.683      0.56632       12.779       2.9056       10.685 


******************/




/***** h=100 new results


OPTMUM Version 3.1.10                                     1/29/2003   1:28 pm
===============================================================================

return code =    0
normal convergence

Value of objective function        0.054201

Parameters    Estimates    Gradient
-----------------------------------------
P01              0.2705       -0.0004
P02              1.6146       -0.0001
P03              3.1369       -0.0002
P04             -0.1997       -0.0003
P05              0.0134        0.0004
P06              0.0075        0.0000
P07             -0.9002       -0.0003
P08             -1.3170        0.0001
P09             -0.3527        0.0005
P10              0.0286        0.0000
P11             -0.4124       -0.0007
P12             -0.0678        0.0008
P13             -0.3775        0.0000
P14              0.2239        0.0000
P15             -1.1164       -0.0002
P16              0.7231       -0.0008
P17              0.5527       -0.0003
P18             -0.2548        0.0000
P19             -1.4163       -0.0001
P20             -0.8357       -0.0003
P21             -0.9222        0.0005
P22             -0.2803        0.0003
P23             -0.6638        0.0001
P24             -1.3273        0.0000
P25             -0.3824        0.0000
P26              1.5732        0.0002
P27             -1.7541       -0.0004
P28              0.1494        0.0001
P29             -0.3856        0.0000
P30             -2.0324       -0.0002
P31              0.4391        0.0001
P32             -0.5324        0.0000
P33             -0.7659        0.0000
P34             -0.3998        0.0001
P35              0.7725       -0.0003
P36              2.0346       -0.0002
P37              1.5410        0.0004
P38              0.3254       -0.0004
P39             -0.2946       -0.0003
P40             -0.0344       -0.0005
P41             -0.1045       -0.0003
P42              0.0937        0.0001
P43              0.8316        0.0001
P44              0.3196       -0.0001
P45              1.0098        0.0002
P46             -0.0384       -0.0005
P47              1.1462        0.0000
P48              2.3443        0.0000
P49              0.1461       -0.0004
P50              0.3191        0.0007
P51             -0.1542       -0.0005
P52              0.1415        0.0000
P53              2.1679        0.0002
P54              2.1420        0.0000
P55              2.4978        0.0003
P56              2.2686       -0.0001
P57              0.3215       -0.0006
P58              1.2159       -0.0001
P59              0.0825        0.0000
P60              1.1300        0.0000
P61              1.8476       -0.0004
P62              1.9529        0.0000
P63              0.3887        0.0000
P64             -0.7484        0.0000
P65              1.8632        0.0000
P66              0.2844        0.0001
P67              1.2156        0.0000
P68              0.3206       -0.0004
P69              0.5641       -0.0001
P70              0.5355        0.0004
P71              0.7014       -0.0001
P72              0.0752       -0.0001
P73              0.9885        0.0002
P74              0.5319       -0.0005
P75              0.3912       -0.0002
P76              0.2571       -0.0003
P77              0.0666       -0.0002
P78             -0.0737        0.0000
P79             -0.5601        0.0003
P80             -0.4743        0.0000
P81             -0.6039        0.0000
P82             -0.8935       -0.0003

Number of iterations      334
Minutes to convergence  2652.21850


Currency markets
          Korea            Indon             Thai            Malay             Aust  

2.411	6.709	18.403	23.583	5.981	
0.429	4.384	1.145	1.046	8.888	
0.223	0.079	0.570	0.454	2.080	
40.467	19.558	35.822	0.758	8.991	
30.754	11.061	0.008	0.771	3.532	
13.606	13.961	7.387	51.400	6.066	
2.922	1.035	7.456	5.938	27.218	
9.188	43.212	29.209	16.051	37.244	

Equity markets
          Korea            Indon             Thai            Malay             Aust               US  

1.209	0.007	19.561	1.378	23.358	0.098	
2.207	32.958	6.186	3.388	10.994	3.915	
47.903	0.278	0.550	2.632	3.972	0.004	
0.164	9.238	23.747	40.390	12.143	0.222	
2.920	8.251	6.790	6.174	12.123	32.297	
45.597	49.113	37.494	42.532	34.122	3.951	
0.000	0.155	5.672	3.506	3.288	59.515	

*******************/


/******h=100

 
===============================================================================
 OPTMUM Version 3.1.10                                     1/31/2003   9:41 am
===============================================================================

return code =    0
normal convergence

Value of objective function        0.053189

Parameters    Estimates    Gradient
-----------------------------------------
P01              0.2637       -0.0003
P02              1.5840       -0.0002
P03              3.1476        0.0000
P04             -0.1824       -0.0001
P05             -0.0963        0.0002
P06              0.0752        0.0002
P07             -0.5978       -0.0003
P08             -0.8661        0.0000
P09             -0.2341        0.0003
P10              0.0354        0.0001
P11             -0.2645       -0.0003
P12             -0.0591        0.0009
P13             -0.4378        0.0002
P14              0.2127        0.0000
P15             -1.1125       -0.0001
P16              0.7217       -0.0005
P17              0.5475        0.0002
P18             -0.2498        0.0002
P19             -1.4453       -0.0001
P20             -0.8405        0.0000
P21             -0.9189        0.0003
P22             -0.2805        0.0004
P23             -0.6547        0.0003
P24             -1.6647       -0.0001
P25             -0.3535       -0.0001
P26              1.5713        0.0002
P27             -1.7513       -0.0001
P28              0.1773        0.0005
P29             -0.3044        0.0001
P30             -1.5933        0.0000
P31              0.3286        0.0000
P32             -0.4437        0.0000
P33             -0.6046        0.0000
P34             -0.3113        0.0000
P35              0.7737       -0.0001
P36              1.7772        0.0001
P37              1.5375        0.0000
P38              0.3266       -0.0002
P39             -0.3018       -0.0001
P40             -0.0303        0.0001
P41             -0.0970       -0.0004
P42              0.1126        0.0001
P43              0.8379        0.0001
P44              0.3081       -0.0002
P45              0.9947        0.0000
P46             -0.0310       -0.0004
P47              0.8429        0.0000
P48              1.8373        0.0000
P49              0.1022        0.0000
P50              0.2514       -0.0001
P51             -0.1215        0.0001
P52              0.0980       -0.0002
P53              1.3987        0.0000
P54              1.4362        0.0001
P55              1.6310        0.0002
P56              1.5083       -0.0001
P57              0.2281       -0.0002
P58              0.9226       -0.0001
P59             -0.0131       -0.0001
P60              1.0464        0.0000
P61              1.5326       -0.0004
P62              1.6159        0.0002
P63              0.2972        0.0000
P64             -0.5629        0.0000
P65              1.4094       -0.0001
P66              0.2175        0.0000
P67              0.9309        0.0000
P68              0.2565       -0.0005
P69              0.3806        0.0000
P70              0.2418        0.0001
P71              0.3864        0.0000
P72             -0.0420        0.0000
P73              0.5397        0.0000
P74              0.3385        0.0003
P75              0.4453        0.0000
P76              0.2626       -0.0001
P77              0.0606        0.0001
P78             -0.0642        0.0000
P79             -0.5427        0.0002
P80             -0.4651       -0.0001
P81             -0.5951        0.0001
P82             -0.8801       -0.0009

Number of iterations        1
Minutes to convergence     1.96302
Currency markets
 US         Korea            Indon             Thai            Malay            
1.408	10.972	32.533	35.382	12.617	
9.737	4.757	3.048	0.063	13.548	
0.078	0.018	0.160	0.146	0.557	
4.310	0.237	56.791	21.744	47.892	
19.360	21.407	0.581	3.197	2.858	
7.124	10.395	2.170	36.177	10.511	
1.564	0.364	3.191	2.903	11.112	
56.419	51.850	1.526	0.387	0.904	

Equity markets
          Korea            Indon             Thai            Malay             Aust               US  

0.731	0.118	11.820	1.602	13.026	0.089	
7.682	25.125	1.939	3.543	5.134	9.587	
46.370	29.198	39.656	1.794	1.195	0.085	
0.796	19.363	37.043	47.884	31.947	5.147	
11.218	0.541	2.505	0.030	3.821	10.591	
33.203	25.618	2.096	41.511	40.231	2.905	
0.000	0.038	4.941	3.637	4.646	71.595	
****/